/* ------------------------------------------------------------------------
 * $Id: UpdateTraversal.cc,v 1.3 2002/09/20 11:21:34 steve Exp $
 *
 * This file is part of 3Dwm: The Three-Dimensional User Environment.
 *
 * 3Dwm: The Three-Dimensional User Environment:
 *	<http://www.3dwm.org>
 *
 * Chalmers Medialab
 * 	<http://www.medialab.chalmers.se>
 * 
 * ------------------------------------------------------------------------
 * File created 2002-03-20 by Niklas Elmqvist.
 *
 * Copyright (c) 2002 Niklas Elmqvist <elm@3dwm.org>.
 * ------------------------------------------------------------------------
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * ------------------------------------------------------------------------
 */

// -- 3Dwm Includes
#include "Nobel/Node.hh"
#include "Celsius/Mutex.hh"
#include "Polhem/NodeImpl.hh"
#include "Polhem/UpdateTraversal.hh"

using namespace Nobel;

// -- Code Segment

UpdateTraversal::UpdateTraversal() 
{
    // empty
}

UpdateTraversal::~UpdateTraversal()
{
    // empty
}

void UpdateTraversal::update(NodeImpl *root)
{
    // Minimize bounding volume updates
    _bv_need_update = maybe;
    
    //cerr << "start -->" << endl;
    // Walk the scene graph!
    root->traverse(Nobel::Visitor_var(_this()));
    //cerr << "end <--" << endl;
} 

void UpdateTraversal::visit(Nobel::Node_ptr n)
{
    // Update the node's bounding volume
    bool result = n->updateBoundingVolume(_bv_need_update != no);
    _bv_need_update = result ? yes : no;
}

void UpdateTraversal::traverse(Nobel::Node_ptr child, Nobel::Transform_ptr t)
{
    // Make sure this is not a nil reference
    if (CORBA::is_nil(child))
	return;

    // Store old BV state and make sure all leaves are updated
    tristate_t last = _bv_need_update;
    _bv_need_update = maybe;
    
    // Perform recursive traversal on the child
    child->traverse(Visitor_var(_this()));
    
    // Update and restore state 
    _bv_need_update = last == no || last == maybe ? _bv_need_update : yes;
}
